/*
	$Id: ExperimentTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataTypes;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Experiment, ExperimentRawBioAssay</dd>
<dt><b>Unused columns</b></dt>
<dd><b>Experiment</b>: addedDate, channels, hasMageml, extraColumnTables,
diskUsage</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li></li>
</ul>
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/

public class ExperimentTransfer extends CommonItemTransfer
{
	RawBioAssayTransfer rbaT = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ExperimentTransfer");

	/**
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	 */
	ExperimentTransfer(UserTransfer userT, GroupTransfer groupT,
			RawBioAssayTransfer rbaT)
	{
		super(userT, groupT);
		log.debug("Checking that necessary transfers have been done before proceeding with Experiment");
		if (rbaT != null && rbaT.isDone())
		{
			this.rbaT = rbaT;
		}
		else
		{
			log.debug("Dependency: RawBioAssays haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring Experiments...");
		int count = getInt("countExperiments");
		initIdMap(count);
		startProgress(count, "Experiments");
		run(prepareStatementFromFile("selectExperiments"));
		done();
		log.info("Transferring Experiments...DONE");
	}

	/**
	Creates a {@link Experiment} from the current position in
	<code>ResultSet</code>.
	
	@param dc {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>Label</code>
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected Experiment createItem(DbControl dc, ResultSet rs)
			throws BaseException, SQLException
	{
		if (log.isDebugEnabled())
		{
			log.debug("Transfers Experiment:" + rs.getString(6) + " with id:" + rs.getInt(1) + " from BASE1");
		}
		Experiment experiment = Experiment.getNew(
				dc, RawDataTypes.getSafeRawDataType("genepix"));
		setCommonItemData(dc, (CommonItem) experiment, rs);
		experiment.setAuthors(rs.getString(9));
		experiment.setAffiliations(rs.getString(10));
		experiment.setExperimentType(rs.getString(11));
		experiment.setExperimentDesign(rs.getString(12));
		experiment.setPublication(rs.getString(13));
		experiment.setAbstract(rs.getString(14));
		experiment.setPubMedId(rs.getString(15));
		try
		{
			experiment.setPublicationDate(rs.getDate(16));
		}
		catch (SQLException e)
		{
			experiment.setPublicationDate(null);
			log.warn("PublicationDate couldn't be set for this Experiment");
			if (log.isDebugEnabled())log.debug("\t" + e);
		}
		addRawBioAssays(dc, experiment, rs.getInt(1));
		if (log.isDebugEnabled())
		{
			log.debug("Transfered Experiment:" + experiment.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");
			log.debug("--------------------------------------------------------------------------");
		}
		return experiment;
	}

	private void addRawBioAssays(DbControl dc, Experiment experiment,
			int experimentId)
	{
		log.info("...adding RawBioAssays");
		PreparedStatement ps = prepareStatementFromFile("selectRawBioAssayByExperiment");
		try
		{
			ps.setInt(1, experimentId);
			ResultSet rs = ps.executeQuery();
			while (rs.next())
			{
				RawBioAssay rba = rbaT.getRawBioAssayById(dc, rs.getInt(1));
				if (rba != null)
				{
					try
					{
						experiment.addRawBioAssay(rba);
						if (log.isDebugEnabled())
						{
							log.debug("RawBioAssay:" + rba.getName() + " added to Experiment:" + experiment.getName());
						}
					}
					catch (Exception e)
					{
						log.warn("Couldn't add RawBioAssay:" + rba.getName() + " to Experiment:" + experiment.getName());
						if (log.isDebugEnabled())log.debug("\t" + e);
					}
				}
			}
		}
		catch (SQLException e)
		{
			log.error("FAILED executing SQL statement", e);
			stop();
		}
	}
}
