/*
	$Id: ExtractTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.Sample;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Extract</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/

public class ExtractTransfer extends MeasuredBioMaterialTransfer
{
	private final SampleTransfer sampleT;
	// private float[] extractQuantities = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ExtractTransfer");
	
	/**
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param protocolT A finished {@link ProtocolTransfer}
	@param sampleT A finished {@link SampleTransfer}
	 */
	ExtractTransfer(UserTransfer userT, GroupTransfer groupT,
			ProtocolTransfer protocolT, SampleTransfer sampleT)
	{
		super(userT, groupT, protocolT);
		log.debug("Checking that necessary transfers have been done before proceeding with Extract");
		this.sampleT = sampleT;
		if (!sampleT.isDone())
		{
			log.debug("Dependency: Samples haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring Extracts...");
		int count = getInt("countExtracts");
		initIdMap(count);
		startProgress(count, "Extracts");
		run(prepareStatementFromFile("selectExtracts"));
		done();
		log.info("Transferring Extracts...DONE");
	}

	/**
	Creates an {@link Extract} from the current position in
	<code>ResultSet</code>.
	
	@param dc {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return Extract
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected Extract createItem(DbControl dc, ResultSet rs)
			throws BaseException, SQLException
	{
		Extract extract = Extract.getNew(dc);
		setMeasuredBioMaterialData(dc, (MeasuredBioMaterial) extract, rs);
		Sample sample = sampleT.getSampleById(dc, rs.getInt(12));
		if (sample != null)
		{
			extract.setSample(sample, rs.getFloat(10));
		}
		if (log.isDebugEnabled())
		{
			log.debug("Transfered Extract: " + extract.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");
		}
		return extract;
	}

	/**
	Returns an extract from the given base1ExtractId.
	
	@param dbc <code>DbControl</code> use to get the extract.
	@param base1ExtractId
	 */
	protected Extract getExtractById(DbControl dbc, int base1ExtractId)
	{
		try
		{
			Integer id = getBase2Id(base1ExtractId);
			return id == null ? null : Extract.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.error("FAILED to get extract in BASE1 by id: " + base1ExtractId, e);
			stop();
		}
		return null;
	}
}
