/*
	$Id: LabelTransfer.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Label;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>Label</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li></li>
</ul>
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/

public class LabelTransfer extends CommonItemTransfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.LabelTransfer");
	
	/**
	Constructs a new LabelTransfer.
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	 */
	LabelTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
	}

	public void start()
	{
		log.info("Transferring Labels...");
		int count = getInt("countLabels");
		initIdMap(count);
		startProgress(count, "Labels");
		run(prepareStatementFromFile("selectLabels"), 0, getBatchSize());
		done();
		log.info("Transferring Labels...DONE");
	}

	/**
	Creates a {@link Label} from the current position in
	<code>ResultSet</code>.
	
	@param dc {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>Label</code>
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected Label createItem(DbControl dc, ResultSet rs)
			throws BaseException, SQLException
	{
		Label label = null;
		String name = rs.getString(6);
		if ("cy3".equals(name) || "cy5".equals(name))
		{
			ItemQuery<Label> query = Label.getQuery();
			query.include(Include.MINE, Include.OTHERS);
			query.restrict(Restrictions.eq(Hql.property("name"), Expressions.string(name)));
			List<Label> result = query.list(dc);
			if (result.size() > 0) label = result.get(0);
		}
		else
		{
			label = Label.getNew(dc);
			setCommonItemData(dc, (CommonItem) label, rs);
		}
		if (log.isDebugEnabled())
		{
			log.debug("Transfered Label: " + label.getName());
			log.debug("--------------------------------------------------------------------------");
		}
		return label;
	}
}
