/**
	$Id: ExportTemplate.java 4548 2008-09-26 12:29:03Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import java.io.IOException;
import java.util.List;

/**
 	A helper interface for the {@link SimpleExport} class to make it
	easier to output exported data in different formats.
	
	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-26 14:29:03 +0200 (Fri, 26 Sep 2008) $
*/
public interface ExportTemplate
{
	/**
		Get the MIME type to assign to files generated by this template.
	 	@return a String 
	*/
	public String getMimeType();
	
	/**
		Start writing the document.
		@throws IOException If there is an error
	*/
	public void beginDocument()
		throws IOException;
	
	/**
		Write headers.
		@throws IOException If there is an error
	*/
	public void writeHeaders()
		throws IOException;
	
	/**
		Start writing a new item.
	 	@param id Id of the new item. 
		@throws IOException If there is an error
	*/
	public void beginItem(int id)
		throws IOException;
	
	/**
		Write annotation values.
	 	@param ep Exported property the annotations belongs to 
	 	@param values Annotation values to be written.
	 	@param unit The unit of the annotation values, or null
	 		to not display write a unit
		@throws IOException If there is an error
	*/
	public void writeAnnotations(ExportedProperty ep, List<?> values, String unit)
		throws IOException;
	
	/**
		Write the specified property values.
	 	@param ep The exported property the <code>data</code> belongs to.
	 	@param data Data to be written.
		@throws IOException If there is an error
	*/
	public void writeProperty(ExportedProperty ep, Object data)
		throws IOException;
	
	/**
		End writing an item
		@throws IOException If there is an error
	*/
	public void endItem()
		throws IOException;
	
	/**
		End writing the document.
		@throws IOException If there is an error
	*/
	public void endDocument()
		throws IOException;
	
}
