/*
	$Id: Path.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.path;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.http.HttpServletRequest;

/**
	This tag defines a navigation path. Usually it is displayed 
	at the top of a page.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;p:path
      id=...
      clazz=...
      style=...
   &gt;
      &lt;p:pathelement ... /&gt;
   &lt;/p:path&gt;
</pre>

	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>yes</td>
		<td>The ID of the path. The value of this attribute
		goes directly into the standard HTML <code>id</code> attribute.
		This means that if you need a reference to the menu object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		menu = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>path</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different menus. If you set the type attribute to horizontal you
			should also change this attribute to <code>menu_horizontal</code>.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different menus. Note! If you want the menu to be hidden to begin with
			you can use: <code>style="display: none;"</code>.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (Thu, 11 Sep 2008) $
*/
public class Path 
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = 6315346458189900215L;

	/**
		The path to the root directory of this webserver.
	*/
	private static String ROOT = null;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = "path";
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;
	
	/**
		Keeps track of the number of path elements so separators can be added.
	*/
	private int numElements;
	
	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}
	
	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}

	String getRoot()
	{
		return ROOT;
	}
	
	private String separator = null;
	
	String getSeparator()
	{
		numElements++;
		if (separator == null)
		{
			separator = "<td class=\"separator\"><img src=\""+getRoot()+"images/pathseparator.gif\"></td>";
		}
		return numElements > 1 ? separator : "";
	}
	
	public int doStartTag()
		throws JspException
	{
		if (ROOT == null) ROOT = ((HttpServletRequest)pageContext.getRequest()).getContextPath()+"/";
		numElements = 0;

		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
		if (getClazz() != null) sb.append(" class=\"").append(getClazz()).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		sb.append("\">\n");
		sb.append("<table border=\"0\" cellspaing=\"0\" cellpadding=\"0\"><tr>\n"); 

		try
		{
			pageContext.getOut().print(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag() 
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		sb.append("</tr></table>");
		sb.append("</div>\n");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}

