/*
	$Id: AccessControlled.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005, 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

/**
	This interface should be implemented by a class representing items
	which requires the logged in user to have permission to access them.
	Since all items in BASE should use access control, this interface should
	be implemented by all items.
	<p>
	The {@link BasicItem} which is the root superclass of all
	items provides an implementation that checks the role keys for
	access permission. Subclasses may override that implementation to
	also check for other things. For example, the {@link OwnedItem}
	class checks the item's owner against the logged in user.

	@author Nicklas
	@version 2.0
*/
public interface AccessControlled
{

	/**
		Check if the logged in user has the desired permission on
		the item.
	  	@param permission The permission to check if the user has.		
		@return TRUE if the user has the permission, FALSE otherwise
	*/
	public boolean hasPermission(Permission permission);

	/**
		Check if the logged in user has the desired permission on
		the item, otherwise throw an exception.
	  	@param permission Permission the user should have. 		
		@throws PermissionDeniedException If the user doesn't have the 
			requested permission
	*/
	public void checkPermission(Permission permission)
		throws PermissionDeniedException;


	/**
		Get the logged in user's permissions on the item.
	*/
	public Set<Permission> getPermissions();


}
