/*
	$Id: AnnotatedItem.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.AnnotatedData;
import net.sf.basedb.core.data.AnnotationSetData;
import net.sf.basedb.info.AnnotatedItemInfo;

/**
	This is a helper class for items that need to implement the
	{@link Annotatable} interface. Most of those items should
	be able to extend this class.

	@author Nicklas
	@version 2.0
  	@param <D> Kind of annotated data the item is annotated with. 
	@see Annotatable
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public abstract class AnnotatedItem<D extends AnnotatedData>
	extends CommonItem<D>
	implements Annotatable
{
	AnnotatedItem(D annotatedData)
	{
		super(annotatedData);
	}

	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	public AnnotationSet getAnnotationSet()
		throws PermissionDeniedException, BaseException
	{
		DbControl dc = getDbControl();
		AnnotationSet as = dc.getItem(AnnotationSet.class, getData().getAnnotationSet(), this);
		if (as == null)
		{
			as = AnnotationSet.getNew(dc, this);
			getData().setAnnotationSet(as.getData());
			dc.saveItemIf(this, as, false);
		}
		return as;
	}
	public boolean isAnnotated()
	{
		return getData().getAnnotationSet() != null;
	}
	public void removeAnnotations()
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.WRITE);
		AnnotationSetData as = getData().getAnnotationSet();
		if (as != null)
		{
			HibernateUtil.deleteData(getDbControl().getHibernateSession(), as);
			getData().setAnnotationSet(null);
		}
	}
	/**
		Default implementation returns null. Should be overriden by subclasses
		that has protocols.
		@return Always null
		@since 2.2
	*/
	public Protocol getProtocol()
	{
		return null;
	}
	// -------------------------------------------

	/**
	 	@since 2.5
	 */
	AnnotatedItemInfo toTransferable(AnnotatedItemInfo info)
	{
		super.toTransferable(info);
		info.setAnnotated(this.isAnnotated());
		return info;
	}
}



