/*
	$Id: BaseException.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.core;

import java.sql.SQLException;

/**
	This is the base class for exceptions in BASE. All other 
	exceptions defined by BASE are subclasses of this one.
	
	@author Nicklas
	@version 2.0
*/
public class BaseException 
	extends RuntimeException
{
	/**
		
	*/
	private static final long serialVersionUID = -3336541323718932298L;

	/**
		Create a new <code>BaseException</code> object.
	*/
	public BaseException()
	{
		super();
	}

	/**
		Create a new <code>BaseException</code> object with the specified 
		<code>message</code>
		@param message The message to throw with the exception
	*/
	public BaseException(String message)
	{
		super(message);
	}
	
	/**
		Create a new <code>BaseException</code> object with the 
		specified <code>message</code>
		@param message The message to throw with the exception
		@param cause The original exception
	*/
	public BaseException(String message, Throwable cause)
	{
		super(message, cause);
	}

	/**
		Create a new <code>BaseException</code> object acting as a 
		wrapper for another exception that caused the error.
		@param cause The original exception
	*/
	public BaseException(Throwable cause)
	{
		this(cause.getMessage(), cause);
	}
	
	/**
		Create a new <code>BaseException</code> object where the
		cause is an SQLException. An SQLException is different from
		other exception in that is uses {@link SQLException#getNextException()}
		instead of {@link Throwable#getCause()} to chain exception. This
		constructor tries to copy the <code>getNextException</code>:s into
		the <code>getCause</code> chain.
		@param cause The SQL exception
	 */
	public BaseException(SQLException cause)
	{
		super(cause.getMessage(), cause);
		while (cause.getNextException() != null && cause.getCause() == null)
		{
			try
			{
				cause.initCause(cause.getNextException());
			}
			catch (IllegalStateException ex)
			{}
			cause = cause.getNextException();
		}
	}
	
}



