/*
	$Id: DiskConsumable.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
	This interface is implemented by items that can use a lot
	of disk space. This interface directs the {@link DbControl}
	object to check the logged in users quota before saving or updating
	items.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public interface DiskConsumable
	extends Controlled, Ownable
{

	/**
		If this item reports the difference in used bytes or the absolute
		number.
		@return TRUE if the item reports the difference, FALSE if it reports the
			absolute
		@see #getBytes()
		@see #getDifference()
	 */
	public boolean useDifference();
	
	/**
		Get the absolute number of bytes this item uses. All items must report this 
		number.
		
		@return The absolute number of bytes the item uses
		@see #useDifference()
		@see #getDifference()
	*/
	public long getBytes();
	
	/**
		Get the number of bytes this items now is using compared to when it was
		loaded from the database. This number may be negative if the item is now
		using less bytes than before. An item doesn't have to report this value
		if {@link #useDifference()} returns FALSE.

		@return The difference in bytes between now and when the item was loaded,
			or 0 if this item doesn't report differences
	*/
	public long getDifference();
	
	
	/**
		Get the {@link Location} the items is stored at.
		@return The location
	*/
	public Location getLocation();
	
	/**
		Get the {@link QuotaType} system id of this item. It is not allowed
		to return the {@link QuotaType#TOTAL} quota type.
		@return The systemid of the QuotaType
	*/
	public String getQuotaTypeSystemId();

}
