/*
	$Id: FileParameterType.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ItemParameterValueData;

/**
	This class represent a parameter type that is a file.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date$
*/
public class FileParameterType
	extends ParameterType<File>
{

	/**
		Create a new file parameter type, allowing null values.
	*/
	public FileParameterType()
	{
		this(null, false, 1);
	}
	
	/**
		Create a new file parameter type.
	 	@param defaultValue Default value of the parameter. Null is allowed
		@param notNull FALSE if nulls values are allowed, TRUE otherwise
	 	@param multiplicity How many values that can be tied to this
			parameter. 0 is unlimited values and 1 is default.
	*/
	public FileParameterType(File defaultValue, boolean notNull, int multiplicity)
	{
		super(File.class, defaultValue, notNull, multiplicity, 0, 0);
	}
	
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "FileParameterType";
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Always validate.
	*/
	@Override
	void validateValue(String name, File value)
		throws InvalidDataException
	{}
	
	/**
		Create a new {@link ItemParameterValueData} object.
	*/
	@Override
	ItemParameterValueData newParameterValueData()
	{
		return new ItemParameterValueData();
	}
	// -------------------------------------------

}
