/*
 $Id: ArrayBatchData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class represents the making of a new batch of microarray
	slides of a particular design.
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.ArrayBatch
	@see <a href="../../../../../../html/developerdoc/api/data_api.arrays.html">Array LIMS - arrays overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`ArrayBatches`" lazy="false"
*/
public class ArrayBatchData 
	extends AnnotatedData
{

	public ArrayBatchData()
	{}

	private ProtocolData protocol;
	/**
		Get the protocol used for printing the arrays.
		@hibernate.many-to-one column="`protocol_id`" not-null="false" outer-join="false"
	*/
	public ProtocolData getProtocol()
	{
		return protocol;
	}
	public void setProtocol(ProtocolData protocol)
	{
		this.protocol = protocol;
	}

	private HardwareData printRobot;
	/**
		Get the hardware used for creating this batch.
		@hibernate.many-to-one column="`hardware_id`" not-null="false" outer-join="false"
	*/
	public HardwareData getPrintRobot()
	{
		return printRobot;
	}
	public void setPrintRobot(HardwareData printRobot)
	{
		this.printRobot = printRobot;
	}

	private ArrayDesignData arrayDesign;
	/**
		Get the array design item that was used for this batch.
		@hibernate.many-to-one column="`arraydesign_id`" not-null="true" update="false" outer-join="false"
	*/
	public ArrayDesignData getArrayDesign()
	{
		return arrayDesign;
	}
	public void setArrayDesign(ArrayDesignData arrayDesign)
	{
		this.arrayDesign = arrayDesign;
	}
	
	private Set<ArraySlideData> arraySlides;
	/**
		This is the inverse end.
		@see ArraySlideData#getArrayBatch()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`arraybatch_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ArraySlideData"
	*/
	Set<ArraySlideData> getArraySlides()
	{
		return arraySlides;
	}
	void setArraySlides(Set<ArraySlideData> arraySlides)
	{
		this.arraySlides = arraySlides;
	}
}
