/*
 $Id: ArraySlideData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

/**
 This class represents a phycsical microarray slide.

 @author enell
 @version 2.0
 @see net.sf.basedb.core.ArraySlide
 @see <a href="../../../../../../html/developerdoc/api/data_api.arrays.html">Array LIMS - arrays overview</a>
 @base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
 @hibernate.class table="`ArraySlides`" lazy="true"
 */
public class ArraySlideData 
	extends AnnotatedData
{

	public ArraySlideData()
	{}

	private int batchIndex;

	/**
		Get the index of this slide in the batch.
		@hibernate.property column="`batch_index`" type="int" not-null="true"
	*/
	public int getBatchIndex()
	{
		return batchIndex;
	}

	public void setBatchIndex(int batchIndex)
	{
		this.batchIndex = batchIndex;
	}

	private boolean destroyed;

	/**
	 Check if the slide is flagged as destroyed or not.
	 @hibernate.property column="`destroyed`" type="boolean" not-null="true"
	 */
	public boolean isDestroyed()
	{
		return destroyed;
	}

	public void setDestroyed(boolean destroyed)
	{
		this.destroyed = destroyed;
	}

	public static final int MAX_BARCODE_LENGTH = 255;
	private String barcode;
	/**
	 Get the barcode of the slide.
	 @hibernate.property column="`barcode`" type="string" length="255" not-null="false"
	 */
	public String getBarcode()
	{
		return barcode;
	}

	public void setBarcode(String barcode)
	{
		this.barcode = barcode;
	}

	private ArrayBatchData arrayBatch;

	/**
	 Get the arrayBatch of this slide.
	 @hibernate.many-to-one column="`arraybatch_id`" not-null="true" update="false"
	 */
	public ArrayBatchData getArrayBatch()
	{
		return arrayBatch;
	}

	public void setArrayBatch(ArrayBatchData arrayBatch)
	{
		this.arrayBatch = arrayBatch;
	}

	private HybridizationData hybridization;

	/**
	 Get the hybridization
	 @hibernate.one-to-one property-ref="arraySlide"
	 */
	public HybridizationData getHybridization()
	{
		return hybridization;
	}

	void setHybridization(HybridizationData hybridization)
	{
		this.hybridization = hybridization;
	}
}
