/*
 $Id: FeatureData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

/**
	This class holds information about a regular feature.
	
	@author enell, Nicklas
	@version 2.0
	@see net.sf.basedb.core.Feature
	@see <a href="../../../../../../html/developerdoc/api/data_api.arrays.html">Array LIMS - arrays overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`Features`" lazy="false"
 */
public class FeatureData
	extends BasicData
	implements BatchableData
{
	FeatureData()
	{}
	
	/**
		Create a new feature which is linked to a well. The reporter of
		the well is copied to the feature.
	*/
	public FeatureData(ArrayDesignBlockData arrayDesignBlock, WellData well)
	{
		setArrayDesignBlock(arrayDesignBlock);
		setWell(well);
		setReporter(well.getReporter());
	}

	/**
		Create a new feature which is linked to a reporter only.
	*/
	public FeatureData(ArrayDesignBlockData arrayDesignBlock, ReporterData reporter)
	{
		setArrayDesignBlock(arrayDesignBlock);
		setWell(null);
		setReporter(reporter);
	}

	private int column;
	/**
		Get the column coordinate of the feature.
		@hibernate.property type="int" update="false"
		@hibernate.column name="`column`" not-null="true"
	*/
	public int getColumn()
	{
		return column;
	}
	public void setColumn(int column)
	{
		this.column = column;
	}

	private int row;

	/**
		Get the row coordinate of the feature.
		@hibernate.property type="int" update="false"
		@hibernate.column name="`row`" not-null="true"
	*/
	public int getRow()
	{
		return row;
	}
	public void setRow(int row)
	{
		this.row = row;
	}

	private int position;

	/**
		Get the position number of the feature. The value is automatically assigned by the core.
		@hibernate.property column="`position`" type="int" not-null="true" update="false"
	*/
	public int getPosition()
	{
		return position;
	}
	/**
		This method was private until 2.6. The value will be overwritten with an
		autogenerated value unless the feature identification method is POSITION.
		The value must be a positive integer.
		@since 2.6
	*/
	public void setPosition(int position)
	{
		this.position = position;
	}

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for this <code>FeatureData</code> item. If given,
		it must be unique per array design.
		@return The features external ID.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false" update="false"
		@since 2.6
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}
	
	private ArrayDesignBlockData arrayDesignBlock;
	/**
		Get the arraydesignblock of this feature. Cannot be public since we don't
		want to expose data objects to clients.

		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`arraydesignblock_id`" not-null="true"
	*/
	ArrayDesignBlockData getArrayDesignBlock()
	{
		return arrayDesignBlock;
	}
	void setArrayDesignBlock(ArrayDesignBlockData arrayDesignBlock)
	{
		this.arrayDesignBlock = arrayDesignBlock;
	}
	
	private ReporterData reporter;
	/**
		Get the reporter of the feature.
		This property is mapped in the hibernate-properties-FeatureData.xml file since
		it must be mapped with a cascade="evict" which is not supported by XDoclet.
	*/
	public ReporterData getReporter()
	{
		return reporter;
	}
	void setReporter(ReporterData reporter)
	{
		this.reporter = reporter;
	}

	private WellData well;
	/**
		Get the well of the feature. Cannot be public since we don't
		want to expose data objects to clients.
		@hibernate.many-to-one column="`well_id`" not-null="false" update="false" outer-join="false"
	*/
	WellData getWell()
	{
		return well;
	}
	void setWell(WellData well)
	{
		this.well = well;
	}
	
}
