/*
  $Id: ImageData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds information about an image produced by scanning a
	hybridization.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Image
	@see <a href="../../../../../../html/developerdoc/api/data_api.rawdata.html">Hybridzations and raw data overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`Images`" lazy="false"
 */
public class ImageData
	extends BasicData
	implements NameableData, RemovableData, FileAttachableData
{
	public ImageData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/	
	private String name;
	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	public String getDescription()
	{
		return description;
	}

	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the FileAttachableData interface
		-------------------------------------------
	*/
	private FileData file;
	public FileData getFile()
	{
		return file;
	}
	public void setFile(FileData file)
	{
		this.file = file;
	}
	// -------------------------------------------

	private ScanData scan;
	/**
		Get the scan that produced this image.
		@hibernate.many-to-one column="`scan_id`" not-null="true" update="false" outer-join="false"
	*/
	public ScanData getScan()
	{
		return scan;
	}
	public void setScan(ScanData scan)
	{
		this.scan = scan;
	}

	private int format;
	/**
		Get the format of the file. 0 = unknown, 1 = TIFF, 2 = JPEG
		@hibernate.property column="`format`" type="int" not-null="true"
	*/
	public int getFormat()
	{
		return format;
	}
	public void setFormat(int format)
	{
		this.format = format;
	}
	
	private boolean preview;
	/**
		A boolean value indicating if this image is a preview image
		or the full-sized image.
		@hibernate.property column="`is_preview`" type="boolean" not-null="true"
	*/
	public boolean isPreview()
	{
		return preview;
	}
	public void setPreview(boolean preview)
	{
		this.preview = preview;
	}

}
