/*
	$Id: MeasuredBioMaterialData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class is the root class for measured biomaterials, ie biomaterials for
	where the quantity is tracked.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.MeasuredBioMaterial
	@see <a href="../../../../../../html/developerdoc/api/data_api.biomaterials.html">Biomaterials overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.subclass discriminator-value="-1" lazy="false"
*/
public abstract class MeasuredBioMaterialData
	extends BioMaterialData
{

	public MeasuredBioMaterialData()
	{}
	
	private BioMaterialData parent;
	/**
		Get the parent biomaterial. Used for non-pooled biomaterials. The
		core must add logic to check that only a biomaterial of the parent
		type is linked.
		@hibernate.many-to-one column="`parent_id`" not-null="false" outer-join="false"
	*/
	public BioMaterialData getParent()
	{
		return parent;
	}
	public void setParent(BioMaterialData parent)
	{
		this.parent = parent;
	}

	private Float originalQuantity;
	/**
		Get the original quantity of the biomaterial.
		@hibernate.property column="`original_quantity`" type="float" not-null="false"
	*/
	public Float getOriginalQuantity()
	{
		return originalQuantity;
	}
	public void setOriginalQuantity(Float originalQuantity)
	{
		this.originalQuantity = originalQuantity;
	}

	private Float remainingQuantity;
	/**
		Get the remaining quantity of the biomaterial.
		@hibernate.property column="`remaining_quantity`" type="float" not-null="false" update="false"
	*/
	public Float getRemainingQuantity()
	{
		return remainingQuantity;
	}
	public void setRemainingQuantity(Float remainingQuantity)
	{
		this.remainingQuantity = remainingQuantity;
	}
	
	private boolean pooled;
	/**
		If the biomaterial was created by pooling other biomaterials of the same type
		or not.
		@hibernate.property column="`pooled`" type="boolean" not-null="false"
	*/
	public boolean isPooled()
	{
		return pooled;
	}
	public void setPooled(boolean pooled)
	{
		this.pooled = pooled;
	}
	private BioMaterialEventData creationEvent;
	/**
		Get the event that created this biomaterial. Mapping is defined
		in hibernate-properties-MeasuredBioMaterialData.xml since XDoclet cannot
		generate it.
	*/
	public BioMaterialEventData getCreationEvent()
	{
		return creationEvent;
	}
	void setCreationEvent(BioMaterialEventData creationEvent)
	{
		this.creationEvent = creationEvent;
	}
	
	private Set<BioMaterialEventData> events;
	/**
		This is the inverse end.
		@see BioMaterialEventData#getBioMaterial()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`biomaterial_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioMaterialEventData"
	*/
	Set<BioMaterialEventData> getEvents()
	{
		return events;
	}
	void setEvents(Set<BioMaterialEventData> events)
	{
		this.events = events;
	}
	
	private Set<BioMaterialSourceEventData> sourceEvents;
	/**
		This is the inverse end.
		@see BioMaterialEventData#getSources()
		@hibernate.set table="`BioMaterialEventSources`" lazy="true" inverse="true"
		@hibernate.collection-key column="`biomaterial_id`"
		@hibernate.collection-composite-element class="net.sf.basedb.core.data.BioMaterialSourceEventData"
		hibernate.collection-many-to-many column="`event_id`" class="net.sf.basedb.core.data.BioMaterialEventData"
	*/
	Set<BioMaterialSourceEventData> getSourceEvents()
	{
		return sourceEvents;
	}
	void setSourceEvents(Set<BioMaterialSourceEventData> sourceEvents)
	{
		this.sourceEvents = sourceEvents;
	}

}
