/*
  $Id: SpotImagesData.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds information about the settiings used to generate spot images
	for a raw bio assay.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.SpotImages
	@see <a href="../../../../../../html/developerdoc/api/data_api.rawdata.html">Hybridzations and raw data overview</a>
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
	@hibernate.class table="`SpotImages`" lazy="false"
 */
public class SpotImagesData
	extends BasicData
{
	public SpotImagesData()
	{}

	private RawBioAssayData rawBioAssay;
	/**
		Get the scan that produced this image.
		@hibernate.many-to-one column="`rawbioassay_id`" not-null="true" unique="true" update="false"
	*/
	public RawBioAssayData getRawBioAssay()
	{
		return rawBioAssay;
	}
	public void setRawBioAssay(RawBioAssayData rawBioAssay)
	{
		this.rawBioAssay = rawBioAssay;
	}

	private int xScale;
	/**
		Get the x-axis scale.
		@hibernate.property column="`x_scale`" type="int" not-null="true"
	*/
	public int getXScale()
	{
		return xScale;
	}
	public void setXScale(int xScale)
	{
		this.xScale = xScale;
	}
	
	private int yScale;
	/**
		Get the y-axis scale.
		@hibernate.property column="`y_scale`" type="int" not-null="true"
	*/
	public int getYScale()
	{
		return yScale;
	}
	public void setYScale(int yScale)
	{
		this.yScale = yScale;
	}
	
	private int xOffset;
	/**
		Get the x-axis offset.
		@hibernate.property column="`x_offset`" type="int" not-null="true"
	*/
	public int getXOffset()
	{
		return xOffset;
	}
	public void setXOffset(int xOffset)
	{
		this.xOffset = xOffset;
	}
	
	private int yOffset;
	/**
		The y-axis offset.
		@hibernate.property column="`y_offset`" type="int" not-null="true"
	*/
	public int getYOffset()
	{
		return yOffset;
	}
	public void setYOffset(int yOffset)
	{
		this.yOffset = yOffset;
	}
	
	private int spotsize;
	/**
		The size of the spots.
		@hibernate.property column="`spotsize`" type="int" not-null="true"
	*/
	public int getSpotsize()
	{
		return spotsize;
	}
	public void setSpotsize(int spotsize)
	{
		this.spotsize = spotsize;
	}
	
	private float quality;
	/**
		The quality setting for the spot images. A value between 0.0 and 1.0.
		@hibernate.property column="`quality`" type="float" not-null="true"
	*/
	public float getQuality()
	{
		return quality;
	}
	public void setQuality(float quality)
	{
		this.quality = quality;
	}
	
	private float gamma;
	/**
		The gamma correction value.
		@hibernate.property column="`gamma`" type="float" not-null="true"
	*/
	public float getGamma()
	{
		return gamma;
	}
	public void setGamma(float gamma)
	{
		this.gamma = gamma;
	}

	private FileData redImageFile;
	/**
		The file used for the red component in the spot images.
		@hibernate.many-to-one column="`redfile_id`" not-null="false" outer-join="false"
	*/
	public FileData getRedImageFile()
	{
		return redImageFile;
	}
	public void setRedImageFile(FileData redImageFile)
	{
		this.redImageFile = redImageFile;
	}

	private FileData greenImageFile;
	/**
		The file used for the green component in the spot images.
		@hibernate.many-to-one column="`greenfile_id`" not-null="false" outer-join="false"
	*/
	public FileData getGreenImageFile()
	{
		return greenImageFile;
	}
	public void setGreenImageFile(FileData greenImageFile)
	{
		this.greenImageFile = greenImageFile;
	}

	private FileData blueImageFile;
	/**
		The file used for the blue component in the spot images.
		@hibernate.many-to-one column="`bluefile_id`" not-null="false" outer-join="false"
	*/
	public FileData getBlueImageFile()
	{
		return blueImageFile;
	}
	public void setBlueImageFile(FileData blueImageFile)
	{
		this.blueImageFile = blueImageFile;
	}

	private FileData spotImagesFile;
	/**
		The file containing the spot images.
		@hibernate.many-to-one column="`spotimagesfile_id`" not-null="false" outer-join="false"
	*/
	public FileData getSpotImagesFile()
	{
		return spotImagesFile;
	}
	public void setSpotImagesFile(FileData spotImagesFile)
	{
		this.spotImagesFile = spotImagesFile;
	}


}
