/*
	$Id: DataFileMetadataReader.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.filehandler;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;

/**
	This interface should be implemented by classes that
	are used to extract metadata from a file of a specific {@link DataFileType}.
	The class to use for validation is specified by the 
	{@link DataFileType#getMetadataReaderClass()} property.
	
	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public interface DataFileMetadataReader
	extends DataFileHandler
{

	/**
		Extract metadata from the file(s) and write the metadata
		to the item passed to the {@link DataFileHandler#setItem(net.sf.basedb.core.FileStoreEnabled)}
		method.
		
		@param dc A DbControl object that can be used for database
			access
	*/
	public void extractMetadata(DbControl dc);

	/**
		Reset all metadata that this class normally can set. This method is called
		if the validation of a file failed. Any previous metadata that may signal
		that a valid file is present should be reset.
		
		@param dc A DbControl object that can be used for database
			access
	*/
	public void resetMetadata(DbControl dc);
}
