/*
	$Id: DataFileValidator.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.filehandler;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidRelationException;
import net.sf.basedb.core.PermissionDeniedException;

/**
	This interface should be implemented by classes that
	are used to validate a file of a specific {@link DataFileType}.
	The class to use for validation is specified by the 
	{@link DataFileType#getValidatorClass()} property.
	
	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public interface DataFileValidator
	extends DataFileHandler
{
	
	/**
		Validate the file(s) that has been added to this validator
		with the {@link DataFileHandler#setFile(net.sf.basedb.core.FileSetMember)}
		method. This method is recommended to throw an {@link InvalidDataException} 
		or a subclass if the file is not a valid file. {@link PermissionDeniedException}:s
		are ignored by the core and will not change the validation status.
		An exception of type {@link InvalidRelationException} is thrown
		the file itself is valid, but a related file that it depends on is not. The
		core will still mark the file as invalid and store the error message,
		but metadata extraction will still be performed.
		
		@param dc A DbControl object that can be used for database
			access
		@throws InvalidDataException If the validator detects an invalid
			file
		@throws InvalidRelationException If the validator detects that the
			file is valid but another file that it depends on is not, metadata
			extract will still happen
	*/
	public void validate(DbControl dc)
		throws InvalidDataException, InvalidRelationException;
}
