/*
	$Id: About.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Samuel Andersson, Johan Enell
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.plugin;

/**
	Get information, such as the name, version and authors,
	about a plugin.

	@author Nicklas
	@version 2.0
	@see Plugin
	@see AboutImpl
*/
public interface About
{
	/**
		The maximum length of the strings returned by the methods
		in this interface that can be stored in the database.
	*/
	public static final int MAX_INFORMATION_LENGTH = 255;

	/**
		The name of the plugin. Mustn't return null.
	*/
	public String getName();

	/**
		The version of the plugin software. May return null.
	*/
	public String getVersion();

	/**
		A description of the plugin and what it does. May return null.
	*/
	public String getDescription();

	/**
		A copyright notice. May return null.
	*/
	public String getCopyright();

	/**
		Contact information to get more information about the plugin.
		May return null.
	*/
	public String getContact();

	/**
		An email adress to get more information about the plugin.
		May return null.
	*/
	public String getEmail();

	/**
		An URL to get more information about the plugin. May return null.
	*/
	public String getUrl();
}
