/*
	$Id: AboutImpl.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.plugin;

/**
	A simple implementation of the {@link About} interface.

	@base.modified $Date$
	@author Enell
	@version 2.0
*/
public class AboutImpl implements About
{
	private final String name;
	private final String description;
	private final String version;
	private final String copyright;
	private final String contact;
	private final String email;
	private final String url;
	
	/**
		Create a new {@link About} object.

		@param name The name of the plugin, null not allowed
		@param description A description of the plugin
		@param version The version of the plugin
		@param copyright A copyright notice
		@param contact Contact information for the plugin
		@param email An email adress to get more information about the plugin
		@param url An URL to get more inforamtion about the plugin
		@throws NullPointerException If the name is null
	*/
	public AboutImpl(String name, String description, String version, String copyright, String contact, String email, String url)
	{
		if (name == null) throw new NullPointerException("The 'name' parameter mustn't be null.");
		this.contact = contact;
		this.copyright = copyright;
		this.description = description;
		this.email = email;
		this.name = name;
		this.url = url;
		this.version = version;
	}

	/*
		From the About interface
		-------------------------------------------
	*/
	public String getContact()
	{
		return contact;
	}

	public String getCopyright()
	{
		return copyright;
	}

	public String getDescription()
	{
		return description;
	}

	public String getEmail()
	{
		return email;
	}

	public String getName()
	{
		return name;
	}

	public String getUrl()
	{
		return url;
	}

	public String getVersion()
	{
		return version;
	}
	// -------------------------------------------
}
