/**
	$Id: QuerySection.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import net.sf.basedb.core.dbengine.DbEngine;

/**
 	Indicates which part of a query that is currently beeing built.
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
	@see Query#getQuerySection()
*/
public enum QuerySection
{
	SELECT
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return false;
		}
	},
	FROM
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return false;
		}
	},
	JOIN
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return false;
		}
	},
	WHERE
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return engine.supportsColumnAliasInWhere();
		}
	},
	GROUPBY
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return engine.supportColumnAliasInGroupBy();
		}
	},
	HAVING
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return engine.supportColumnAliasInHaving();
		}
	},
	ORDERBY
	{
		public boolean supportsColumnAlias(DbEngine engine)
		{
			return engine.supportColumnAliasInHaving();
		}
	};
	
	/**
	 	If the current database supports column aliases in that part of the query.
	 	@param engine  
	 	@return TRUE if column alias is supported, FALSE otherwise.
	 */
	public abstract boolean supportsColumnAlias(DbEngine engine);
	
}
