/*
	$Id: Select.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

/**
	A selection query element. This is a query element that can appear in 
	the <code>SELECT</code> part of a {@link Query}. Typically a selection
	consists of the name of a property or column and an alias. For example:
	<pre class="code">
SELECT item.name AS name ...
</pre>
	
	The {@link Selects} factory class contains some methods for creating
	selection elements.

	@author Nicklas
	@version 2.0
	@see Selects
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public interface Select
	extends QueryElement
{

	/**
		Get the alias of the selected expression.
		@return The alias, or null if no alias has been specified
	*/
	public String getAlias();
	
	/**
		Get the expression object representing the selected expression.
		@return An expression object
	*/
	public Expression getExpression();
	
	/**
		If this expression should be use in count distinct queries or not.
		<pre class="code">
SELECT COUNT(DISTINCT expression)
</pre>
		@return TRUE or FALSE
	*/
	public boolean useInDistinctCount();

}
