/*
	$Id: PositionFunction.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Johan Enell
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;

import org.nfunk.jep.Node;
import org.nfunk.jep.ParseException;

import java.util.Stack;


/**
	A JEP function class that adds a <code>pos</code> function to a 
	JEP expression parser. The function will look up position.
	For example:
	<code>pos()</code>
	<p>
	To be able to use this function it must be registered with the JEP
	parser. This function can only be used to convert a JEP expression to
	a Query API {@link Expression}. It doesn't support dynamic evalution.
	
	@author Enell
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
	@see Jep
	@see net.sf.basedb.util.BioAssaySetUtil#createJepExpression(net.sf.basedb.core.DbControl, net.sf.basedb.core.BioAssaySet, String, boolean)
*/
public class PositionFunction 
	implements JepExpressionFunction
{

	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "pos"
	*/
	public String getFunctionName()
	{
		return "pos";
	}
	// -------------------------------------------
	/*
		From the JepExpressionFunction interface
		-------------------------------------------
	*/
	/**
		Use the {@link Dynamic#column(VirtualColumn)} method to create an
		expression referencing a position.
	*/
	public Expression toExpression(Node node)
	{
		return Dynamic.column(VirtualColumn.POSITION);
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/

	/**
		@return Always 0
	*/
	public int getNumberOfParameters()
	{
		return 0;
	}
	/**
		Can't be changed.
	*/
	public void setCurNumberOfParameters(int n) 
	{}
	
	public boolean checkNumberOfParameters(int n)
	{
		return n==0;
	}
	
	@SuppressWarnings("unchecked")
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null || stack.empty()) 
		{
			throw new ParseException("Stack is empty");
		}
		Object position = stack.pop();
		if (position instanceof Number)
		{
			stack.push(position);
		}
		else
		{
			throw new ParseException("Invalid parameter type: " + position + "; expected integer");
		}
	}
	// -------------------------------------------
}
