/**
	$Id: AnnotationTypeInfo.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.info;

import java.io.Serializable;


/**
  	An information class that holds information about an 
 	{@link net.sf.basedb.core.AnnotationType}.
 	This class only holds simple objects that can
 	be handled by web services.
  	@author Nicklas
  	@version 2.5 	
*/
public class AnnotationTypeInfo 
	extends CommonItemInfo 
	implements Serializable
{

	private static final long serialVersionUID = 4294524681521272236L;
	private String externalId;
	private String defaultValue;
	private int multiplicity;
	private String valueType;
	private boolean enumeration;
	private boolean protocolParameter;
	private boolean requiredForMiame;
	private String[] enumValues;
	private String[] itemTypes;
	
	/**
	 	Creates information object with 
	 	no properties set.
	*/
	public AnnotationTypeInfo()
	{
		super();
	}

	/**
	 	@return Returns the default value
	*/
	public String getDefaultValue()
	{
		return defaultValue;
	}

	/**
	 	@param defaultValue The default value to set
	*/
	public void setDefaultValue(String defaultValue)
	{
		this.defaultValue = defaultValue;
	}

	/**
	 	@return TRUE if the annotation type is an enumeration
	*/
	public boolean isEnumeration()
	{
		return enumeration;
	}

	/**
		@param enumeration TRUE if the annotation type is an enumeration
	*/
	public void setEnumeration(boolean enumeration)
	{
		this.enumeration = enumeration;
	}

	/**
	 	@return Returns the possible values to select from if this annotation type is an enumeration
	*/
	public String[] getEnumValues()
	{
		return enumValues;
	}

	/**
		@param enumValues The possible values to select from
	*/
	public void setEnumValues(String[] enumValues)
	{
		this.enumValues = enumValues;
	}

	/**
		@return The external ID of the annotation type or null
	*/
	public String getExternalId()
	{
		return externalId;
	}

	/**
		@param externalId The externalId to set
	*/
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	/**
		@return The maximum number of allowed values or 0 to allow any number of values
	*/
	public int getMultiplicity()
	{
		return multiplicity;
	}

	/**
		@param multiplicity The number of values that are allowed when annotating items
	*/
	public void setMultiplicity(int multiplicity)
	{
		this.multiplicity = multiplicity;
	}

	/**
		@return TRUE if this annotation type is a protocol parameter, FALSE if it is a
	 		regular annotation
	*/
	public boolean isProtocolParameter()
	{
		return protocolParameter;
	}

	/**
		@param protocolParameter TRUE if this annotation type is a protocol parameter, FALSE if it is a
	 		regular annotation
	*/
	public void setProtocolParameter(boolean protocolParameter)
	{
		this.protocolParameter = protocolParameter;
	}

	/**
		@return TRUE if values must be specified for this annotation type in order
			to be MIAME compliant
	*/
	public boolean isRequiredForMiame()
	{
		return requiredForMiame;
	}

	/**
		@param requiredForMiame TRUE if values must be specified for this annotation type in order
			to be MIAME compliant
	*/
	public void setRequiredForMiame(boolean requiredForMiame)
	{
		this.requiredForMiame = requiredForMiame;
	}

	/**
		@return The class of values that can be used for annotations. The value
			is the string representation of one of the options in {@link net.sf.basedb.core.Type},
			ie. STRING, INT, LONG
	*/
	public String getValueType()
	{
		return valueType;
	}

	/**
	 	@param valueType The class of values that can be used for annotations.
	 	@see #getValueType()
	*/
	public void setValueType(String valueType)
	{
		this.valueType = valueType;
	}
	
	/**
		@return The types of items that can be annotated with this annotation type. The value
			is the string representation of one of the options in {@link net.sf.basedb.core.Item},
			ie. SAMPLE, EXTRACT, BIOASSAYSET, RAWBIOASSAY, etc.
	*/
	public String[] getItemTypes()
	{
		return itemTypes;
	}
	
	/**
	 	@param itemTypes The type of items that can be annotated with this annotation type.
	 	@see #getItemTypes()
	*/
	public void setItemTypes(String[] itemTypes)
	{
		this.itemTypes = itemTypes;
	}


}
