/*
	$Id: TestItemImporter.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Response;


public class TestItemImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing item import using plugin");
		
		// Create referenced items
		int samplingProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.SAMPLING), "Sampling", false);
		int extractionProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.EXTRACTION), "Extraction", false);
		int labelingProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.LABELING), "Labeling", false);
		int poolingProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.POOLING), "Pooling", false);
		int hybProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.HYBRIDIZATION), "Hybridization", false);
		int hybStationId = TestHardware.test_create(SystemItems.getId(HardwareType.HYBRIDIZATION_STATION), "Hybstation", false);
		int scanProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.SCANNING), "Scanning", false);
		int scannerId = TestHardware.test_create(SystemItems.getId(HardwareType.SCANNER), "Scanner", false);
		int featureProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.FEATURE_EXTRACTION), "Feature", false);
		int featureSoftwareId = TestSoftware.test_create(SystemItems.getId(SoftwareType.FEATURE_EXTRACTION), "Feature", false);
		int genericId = TestPlatform.test_get(Platform.GENERIC);
		int variantId = TestPlatform.test_create_variant(genericId, "Variant", "test.variant", null, 0, null);
		int robotId = TestHardware.test_create(SystemItems.getId(HardwareType.PRINT_ROBOT), "Robot", false);
		int printProtocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.PRINTING), "Printing", false);
		
		// Import biosources
		int bioSourceFileId = TestFile.test_create("data/test.batchimport.biosources.txt", false, false);
		int bioSourceImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.BioSourceImporter");
		int bioSourceJobId = test_create_item_import_job(Item.BIOSOURCE, bioSourceImporterId, bioSourceFileId);
		ok &= TestJob.test_execute(bioSourceJobId);
		TestBioSource.test_list(3);
		
		// Import samples
		int sampleFileId = TestFile.test_create("data/test.batchimport.samples.txt", false, false);
		int sampleImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.SampleImporter");
		int sampleJobId = test_create_item_import_job(Item.SAMPLE, sampleImporterId, sampleFileId);
		ok &= TestJob.test_execute(sampleJobId);
		TestSample.test_list(4);

		// Import extracts
		int extractFileId = TestFile.test_create("data/test.batchimport.extracts.txt", false, false);
		int extractImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ExtractImporter");
		int extractJobId = test_create_item_import_job(Item.EXTRACT, extractImporterId, extractFileId);
		ok &= TestJob.test_execute(extractJobId);
		TestExtract.test_list(4);

		// Import labeled extracts
		int labledExtractFileId = TestFile.test_create("data/test.batchimport.labeledextracts.txt", false, false);
		int labledExtractImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.LabeledExtractImporter");
		int labledExtractJobId = test_create_item_import_job(Item.LABELEDEXTRACT, labledExtractImporterId, labledExtractFileId);
		ok &= TestJob.test_execute(labledExtractJobId);
		TestLabeledExtract.test_list(7);
		
		// Import array designs
		int cdfId = TestFile.test_create("data/test.affymetrix.cdf", false, false);
		TestFile.test_rename(cdfId, "test.cdf");
		int mapId = TestFile.test_create("data/test.reportermap.import.txt", false, false);
		TestFile.test_rename(mapId, "test.map");
		int designFileId = TestFile.test_create("data/test.batchimport.arraydesigns.txt", false, false);
		int designImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArrayDesignImporter");
		int designJobId = test_create_item_import_job(Item.ARRAYDESIGN, designImporterId, designFileId);
		ok &= TestJob.test_execute(designJobId);
		TestArrayDesign.test_list(3);
		
		// Import array batches
		int arrayBatchFileId = TestFile.test_create("data/test.batchimport.arraybatches.txt", false, false);
		int arrayBatchImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArrayBatchImporter");
		int arrayBatchJobId = test_create_item_import_job(Item.ARRAYBATCH, arrayBatchImporterId, arrayBatchFileId);
		ok &= TestJob.test_execute(arrayBatchJobId);
		TestArrayBatch.test_list(3);

		// Import array slides
		int arraySlideFileId = TestFile.test_create("data/test.batchimport.arrayslides.txt", false, false);
		int arraySlideImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ArraySlideImporter");
		int arraySlideJobId = test_create_item_import_job(Item.ARRAYSLIDE, arraySlideImporterId, arraySlideFileId);
		ok &= TestJob.test_execute(arraySlideJobId);
		TestArraySlide.test_list(5);
		
		// Import hybridizations
		int hybFileId = TestFile.test_create("data/test.batchimport.hybridizations.txt", false, false);
		int hybImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.HybridizationImporter");
		int hybJobId = test_create_item_import_job(Item.HYBRIDIZATION, hybImporterId, hybFileId);
		ok &= TestJob.test_execute(hybJobId);
		TestHybridization.test_list(3);
		
		// Import scans
		int scanFileId = TestFile.test_create("data/test.batchimport.scans.txt", false, false);
		int scanImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.ScanImporter");
		int scanJobId = test_create_item_import_job(Item.SCAN, scanImporterId, scanFileId);
		ok &= TestJob.test_execute(scanJobId);
		TestScan.test_list(3);
	
		// Import raw bioassays
		int celId = TestFile.test_create("data/test.affymetrix.cel", false, false);
		TestFile.test_rename(celId, "test.cel");
		int gprId = TestFile.test_create("data/test.rawdata.import.txt", false, false);
		TestFile.test_rename(gprId, "test.gpr");
		int rbaFileId = TestFile.test_create("data/test.batchimport.rawbioassays.txt", false, false);
		int rbaImporterId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.batchimport.RawBioAssayImporter");
		int rbaJobId = test_create_item_import_job(Item.RAWBIOASSAY, rbaImporterId, rbaFileId);
		ok &= TestJob.test_execute(rbaJobId);
		TestRawBioAssay.test_list(4);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		// Delete raw bioassays
		TestRawBioAssay.test_delete_all();
		TestJob.test_delete(rbaJobId);
		TestFile.test_delete(rbaFileId);
		TestFile.test_delete(celId);
		TestFile.test_delete(gprId);
		
		// Delete scans
		TestScan.test_delete_all();
		TestJob.test_delete(scanJobId);
		TestFile.test_delete(scanFileId);

		// Delete hybridizations
		TestHybridization.test_delete_all();
		TestJob.test_delete(hybJobId);
		TestFile.test_delete(hybFileId);

		// Delete array slides
		TestArraySlide.test_delete_all();
		TestJob.test_delete(arraySlideJobId);
		TestFile.test_delete(arraySlideFileId);
		
		// Delete array batches
		TestArrayBatch.test_delete_all();
		TestJob.test_delete(arrayBatchJobId);
		TestFile.test_delete(arrayBatchFileId);
		
		// Delete array designs
		TestArrayDesign.test_delete_all();
		TestJob.test_delete(designJobId);
		TestFile.test_delete(designFileId);
		TestFile.test_delete(cdfId);
		TestFile.test_delete(mapId);
		
		// Delete labeled extracts
		TestLabeledExtract.test_delete_all();
		TestJob.test_delete(labledExtractJobId);
		TestFile.test_delete(labledExtractFileId);

		// Delete extracts
		TestExtract.test_delete_all();
		TestJob.test_delete(extractJobId);
		TestFile.test_delete(extractFileId);
		
		// Delete samples
		TestSample.test_delete_all();
		TestJob.test_delete(sampleJobId);
		TestFile.test_delete(sampleFileId);

		// Delete biosources
		TestBioSource.test_delete_all();
		TestJob.test_delete(bioSourceJobId);
		TestFile.test_delete(bioSourceFileId);

		// References items
		TestProtocol.test_delete(printProtocolId);
		TestHardware.test_delete(robotId);
		TestProtocol.test_delete(samplingProtocolId);
		TestProtocol.test_delete(extractionProtocolId);
		TestProtocol.test_delete(labelingProtocolId);
		TestProtocol.test_delete(poolingProtocolId);
		TestProtocol.test_delete(hybProtocolId);
		TestHardware.test_delete(hybStationId);
		TestProtocol.test_delete(scanProtocolId);
		TestHardware.test_delete(scannerId);
		TestProtocol.test_delete(featureProtocolId);
		TestSoftware.test_delete(featureSoftwareId);
		TestPlatform.test_delete_variant(variantId);

		write("++Testing item import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_item_import_job(Item itemType, int pluginDefinitionId, int fileId)
	{
		if (pluginDefinitionId == 0 || fileId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition plugin = PluginDefinition.getById(dc, pluginDefinitionId);
			Job j = Job.getNew(dc, plugin, null, null);
			j.setName("Import " + itemType.toString());
			
			// Step 1 - file and parser regular expressions
			PluginConfigurationRequest request = j.configure(new GuiContext(itemType, GuiContext.Type.LIST));
			//write_request_information(request.getRequestInformation());
			request.getRequestInformation();
			
			request.setParameterValue("file", File.getById(dc, fileId));
			setParserParameters(request);
			setColumnMappingParameters(request, itemType);
			PluginResponse response = request.invoke();
			if (response.getStatus() != Response.Status.CONTINUE)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}

			// Step 2 - other options and error handling
			request = response.getNextRequest();
			request.getRequestInformation();
			setErrorHandlingParameters(request, itemType);
			response = request.invoke();
			
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for " + itemType + " import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for " + itemType + " import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
	
	private static void setParserParameters(PluginConfigurationRequest request)
	{
		request.setParameterValue("dataSplitterRegexp", "\\t");
		request.setParameterValue("dataHeaderRegexp", "Name.*");
		request.setParameterValue("ignoreRegexp", "#.*");
		
		request.setParameterValue("idMethod", "name");
		request.setParameterValue("mode", "create+update");
	}
	
	private static void setColumnMappingParameters(PluginConfigurationRequest request, Item itemType)
	{
		request.setParameterValue("nameColumnMapping", "\\Name\\");
		request.setParameterValue("descriptionColumnMapping", "\\Description\\");
		
		if (itemType == Item.SAMPLE || itemType == Item.EXTRACT || itemType == Item.LABELEDEXTRACT)
		{
			request.setParameterValue("externalIdColumnMapping", "\\External ID\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("createdColumnMapping", "\\Created\\");
			request.setParameterValue("originalQuantityColumnMapping", "\\Original quantity\\");
			request.setParameterValue("pooledColumnMapping", "\\Pooled\\");
			request.setParameterValue("parentColumnMapping", "\\Parent\\");
			request.setParameterValue("usedQuantityColumnMapping", "\\Used quantity\\");
		}
		if (itemType == Item.LABELEDEXTRACT)
		{
			request.setParameterValue("labelColumnMapping", "\\Label\\");
		}
		if (itemType == Item.HYBRIDIZATION)
		{
			request.setParameterValue("numArraysColumnMapping", "\\Num arrays\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("hardwareColumnMapping", "\\Hyb station\\");
			request.setParameterValue("createdColumnMapping", "\\Created\\");
			request.setParameterValue("parentColumnMapping", "\\Parent\\");
			request.setParameterValue("usedQuantityColumnMapping", "\\Used quantity\\");
			request.setParameterValue("arrayIndexColumnMapping", "\\Array index\\");
			request.setParameterValue("arraySlideColumnMapping", "\\Array slide\\");
		}
		if (itemType == Item.SCAN)
		{
			request.setParameterValue("hybridizationColumnMapping", "\\Hybridization\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("hardwareColumnMapping", "\\Scanner\\");
		}
		if (itemType == Item.RAWBIOASSAY)
		{
			request.setParameterValue("dataDirectory", "/");
			request.setParameterValue("arrayIndexColumnMapping", "\\Array index\\");
			request.setParameterValue("scanColumnMapping", "\\Scan\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("platformColumnMapping", "\\Platform\\");
			request.setParameterValue("variantColumnMapping", "\\Variant\\");
			request.setParameterValue("rawDataTypeColumnMapping", "\\Raw data type\\");
			request.setParameterValue("softwareColumnMapping", "\\Software\\");
			request.setParameterValue("arrayDesignColumnMapping", "\\Array design\\");
			request.setParameterValue("fileTypeColumnMapping", "\\File type\\");
			request.setParameterValue("fileColumnMapping", "\\File\\");
		}
		if (itemType == Item.ARRAYDESIGN)
		{
			request.setParameterValue("dataDirectory", "/");
			request.setParameterValue("numArraysColumnMapping", "\\Num arrays\\");
			request.setParameterValue("platformColumnMapping", "\\Platform\\");
			request.setParameterValue("variantColumnMapping", "\\Variant\\");
			request.setParameterValue("fileTypeColumnMapping", "\\File type\\");
			request.setParameterValue("fileColumnMapping", "\\File\\");
		}
		if (itemType == Item.ARRAYBATCH)
		{
			request.setParameterValue("arrayDesignColumnMapping", "\\Array design\\");
			request.setParameterValue("protocolColumnMapping", "\\Protocol\\");
			request.setParameterValue("hardwareColumnMapping", "\\Robot\\");
		}
		if (itemType == Item.ARRAYSLIDE)
		{
			request.setParameterValue("arrayBatchColumnMapping", "\\Array batch\\");
			request.setParameterValue("indexColumnMapping", "\\Index\\");
			request.setParameterValue("barcodeColumnMapping", "\\Barcode\\");
			request.setParameterValue("destroyedColumnMapping", "\\Destroyed\\");
		}
	}
	
	private static void setErrorHandlingParameters(PluginConfigurationRequest request, Item itemType)
	{
		request.setParameterValue("defaultError", "fail");
		request.setParameterValue("stringTooLongError", "crop");
		request.setParameterValue("numberFormatError", "skip");
		request.setParameterValue("numberOutOfRangeError", "skip");
		
		if (itemType == Item.SAMPLE)
		{
			//request.setParameterValue("dryRun", true);
		}
		request.setParameterValue("logFile", "/" + itemType.name() + ".log");
	}
	
}

