import net.sf.basedb.core.Path;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

public class TestPath
{
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	
	private static boolean ok = true;

	public static Boolean test_all()
	{
		write("++Testing path");
		test_create("/filename", Path.Type.FILE, false);
		test_create("/directory/filename", Path.Type.FILE, false);
		test_create("/directory/subdirectory/filename", Path.Type.FILE, false);
		test_create("~userlogin/filename", Path.Type.FILE, false);
		test_create("~userlogin/directory/filename", Path.Type.FILE, false);
		test_create("~userlogin/directory/subdirectory/filename", Path.Type.FILE, false);
		test_create("~/filename", Path.Type.FILE, false);
		test_create("~/directory/filename", Path.Type.FILE, false);
		test_create("~/directory/subdirectory/filename", Path.Type.FILE, false);

		test_create("/", Path.Type.DIRECTORY, false);
		test_create("/directory", Path.Type.DIRECTORY, false);
		test_create("/directory/subdirectory", Path.Type.DIRECTORY, false);
		test_create("/directory/subdirectory/subsub", Path.Type.DIRECTORY, false);
		test_create("~userlogin", Path.Type.DIRECTORY, false);
		test_create("~userlogin/directory", Path.Type.DIRECTORY, false);
		test_create("~userlogin/directory/subdirectory", Path.Type.DIRECTORY, false);
		test_create("~userlogin/directory/subdirectory/subsub", Path.Type.DIRECTORY, false);
		test_create("~", Path.Type.DIRECTORY, false);
		test_create("~/directory", Path.Type.DIRECTORY, false);
		test_create("~/directory/subdirectory", Path.Type.DIRECTORY, false);
		test_create("~/directory/subdirectory/subsub", Path.Type.DIRECTORY, false);

		test_valid_name("abc", true);
		test_valid_name("~abc", false);
		test_valid_name("\\abc", false);
		test_valid_name("/abc", false);
		test_valid_name(":abc", false);
		test_valid_name(";abc", false);
		test_valid_name("*abc", false);
		test_valid_name("?abc", false);
		test_valid_name("<abc", false);
		test_valid_name(">abc", false);
		test_valid_name("|abc", false);
		
		test_create("/file<name>", Path.Type.FILE, true);
		test_create("/dire<ctory>/filename", Path.Type.FILE, true);
		test_create("/directory/file<name>", Path.Type.FILE, true);
		test_create("/sub<dir>", Path.Type.DIRECTORY, true);
		test_create("/directory/sub<dir>", Path.Type.DIRECTORY, true);
		
		test_replace_invalid("abc", ".", "abc");
		test_replace_invalid("~abc", ".", ".abc");
		test_replace_invalid("~:a*?bc<>", ".", ".a.bc.");

		write("++Testing path "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static void test_create(String path, Path.Type type, boolean shouldFail)
	{
		try
		{
			Path p = new Path(path, type);
			String pp = p.toString();
			if (!path.equals(pp))
			{
				throw new Exception("Original path ("+path+") is different from path: "+pp);
			}
			if (shouldFail)
			{
				write("--Path '"+path+"' FAILED");
				new Throwable("Expected test to fail but it didn't.").printStackTrace();
				ok = false;
			}
			else
			{
				write("--Path " + p.getType() + ": '"+p+"' OK");
			}
		}
		catch (Throwable ex)
		{
			if (!shouldFail)
			{
				write("--Path '"+path+"' FAILED");
				ex.printStackTrace();
				ok = false;
			}
			else
			{
				write("--Path " + type + ": '"+path+"' OK");
			}
		}
	}
	
	static void test_valid_name(String name, boolean is_valid)
	{
		try
		{
			boolean name_is_valid = Path.isValidName(name);
			if (name_is_valid == is_valid)
			{
				write("--Name '"+name+"' is "+(name_is_valid ? "valid" : "invalid")+" OK");
			}
			else
			{
				write("--Name '"+name+"' is "+(name_is_valid ? "valid" : "invalid")+" FAILED");
				ok = false;
			}
		}
		catch (Throwable ex)
		{
			write("--Name '"+name+"' FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void test_replace_invalid(String invalidName, String replacement, String validName)
	{
		try
		{
			String fixedName = Path.makeSafeFilename(invalidName, replacement);
			if (validName.equals(fixedName))
			{
				write("--Name '"+invalidName+"' changed to " +fixedName + ": OK");
			}
			else
			{
				write("--Name '"+invalidName+"' changed to " +fixedName + ": FAILED");
				ok = false;
			}
		}
		catch (Throwable ex)
		{
			write("--Replace invalid name '"+invalidName+"' FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	
	static void write(String message)
	{
		System.out.println(message);
	}
}
