/*
	$Id: TestReporterList.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class TestReporterList
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing reporter lists");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create("net.sf.basedb.clients.test.reporterlist1", true);
		int id2 = test_create("net.sf.basedb.clients.test.reporterlist2", false);
		int id3 = test_create("net.sf.basedb.clients.test.reporterlist3", false);
		int id4 = test_create("net.sf.basedb.clients.test.reporterlist4", false);
		int id5 = test_create("net.sf.basedb.clients.test.reporterlist5", false);
		int id6 = test_create("net.sf.basedb.clients.test.reporterlist6", false);
		test_load(id1);
		test_list(-1);

		TestReporter.test_import_from_file("data/test.rawdata.import.txt", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);

		test_add_reporters(id1, 20, 0);
		test_list_reporters(id1, 20);
		test_add_reporters(id2, 20, 15); // 5 reporters in common with 'id1', 15 new

		test_union(id3, 35, id1, id2); // 'id3' is now the union of 'id1' and 'id2'
		test_intersect(id4, 5, id1, id2); // 'id4' is now the intersection of 'id1' and 'id2'
		test_remove(id5, id3, 30, id4); // 'id5' is now the complement of 'id3' to 'id4'
		test_retain(id6, id2, 5, id1); // 'id6' is now the intersection on 'id1' and 'id2'
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		test_delete(id3);
		test_delete(id4);
		test_delete(id5);
		test_delete(id6);
		TestReporter.test_delete();
		write("++Testing reporter lists "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String externalId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.REPORTERLIST)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getNew(dc);
			rl.setExternalId(externalId);
			if (setAll)
			{
				rl.setName("Test reporter list");
				rl.setDescription("Added at "+new Date());
			}
			dc.saveItem(rl);
			dc.commit();
			id = rl.getId();
			write_item(0, rl);
			write("--Create reporter list OK");
		}
		catch (Throwable ex)
		{
			write("--Create reporter list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getById(dc, id);
			write_item(0, rl);
			write("--Load reporter list OK");
		}
		catch (Throwable ex)
		{
			write("--Load reporter list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<ReporterList> q = ReporterList.getQuery();
			ItemResultList<ReporterList> l = q.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List reporter lists OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List reporter lists FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getById(dc, id);
			dc.deleteItem(rl);
			dc.commit();
			write("--Delete reporter list OK");
		}
		catch (Throwable ex)
		{
			write("--Delete reporter list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal ID");
			write("-- \t-- \t--------- \t-----------\t-----------");
		}
	}
	static void write_item(int i, ReporterList rl)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+rl.getId()+"\t"+rl.getName()+"\t"+
			rl.getDescription()+"\t"+rl.getExternalId());
	}
	static void write_item(int i, ReporterData r, Float score)
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+r.getExternalId()+"\t"+score);
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_add_reporters(int id, int maxNumber, int first)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getById(dc, id);
			
			write_item(0, rl);
			DataQuery<ReporterData> query = Reporter.getQuery();
			query.order(Orders.asc(Hql.property("externalId")));
			query.setMaxResults(maxNumber);
			query.setFirstResult(first);
			DataResultIterator<ReporterData> reporters = query.iterate(dc);
			int i = 0;
			while (reporters.hasNext() && i < maxNumber)
			{
				ReporterData r = reporters.next();
				Float score = (float)Math.random() * 50000;
				rl.addReporter(r, score);
				write_item(i, r, score);
				i++;
			}
			reporters.close();
			dc.commit();
			
			write("--Add reporters to list OK (" + i + ")");
		}
		catch (Throwable ex)
		{
			write("--Add reporters to list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_reporters(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getById(dc, id);
			
			ReporterScoreQuery query = rl.getReporterScores();
			ReporterScoreResultIterator reporters = query.iterate(dc);
			
			int i = 0;
			while (reporters.hasNext())
			{
				ReporterScore r = reporters.next();
				write_item(i, r.getReporter(), r.getScore());
				i++;
			}
			if (expectedResults >= 0 && expectedResults != i)
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+i);
			}
			write("--List reporters in list OK (" + i + ")");
		}
		catch (Throwable ex)
		{
			write("--List reporters in list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_setExperiment(int id, int experimentId)
	{
		if (id == 0 || experimentId == 0)return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList rl = ReporterList.getById(dc, id);
			Experiment exp = Experiment.getById(dc, experimentId);
			
			rl.setExperiment(exp);
			dc.commit();
		}
		catch (Throwable ex)
		{
			write ("--Set experiment to list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_union(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList to = ReporterList.getById(dc, toId);
			List<ReporterList> from = new ArrayList<ReporterList>(fromId.length);
			for (int rlId : fromId)
			{
				from.add(ReporterList.getById(dc, rlId));
			}
			to.addUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create union OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create union FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_intersect(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList to = ReporterList.getById(dc, toId);
			List<ReporterList> from = new ArrayList<ReporterList>(fromId.length);
			for (int rlId : fromId)
			{
				from.add(ReporterList.getById(dc, rlId));
			}
			to.addIntersection(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create intersection OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create intersection FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_remove(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList to = ReporterList.getById(dc, toId);
			to.addAll(ReporterList.getById(dc, baseId));
			List<ReporterList> from = new ArrayList<ReporterList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(ReporterList.getById(dc, rlId));
			}
			to.removeUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Remove reporters OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Remove reporters FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_retain(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterList to = ReporterList.getById(dc, toId);
			to.addAll(ReporterList.getById(dc, baseId));
			List<ReporterList> from = new ArrayList<ReporterList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(ReporterList.getById(dc, rlId));
			}
			to.retainUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Retain reporters OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Retain reporters FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}

