/**
	$Id: AnnotationTypeService.java 4513 2008-09-11 20:06:36Z jari $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.QueryOptions;


/**
 	Web service class for annotation types.
 	{@link net.sf.basedb.core.AnnotationType}.
  	@author Nicklas
  	@version 2.5
*/
public class AnnotationTypeService
	extends AbstractRPCService
{
	/**
	 	
	 */
	public AnnotationTypeService()
	{}
	
	/**
		Get information about an annotation type that you know the ID of.
		@param ID Id to identify the session.
		@param itemId The ID of the annotation type
		@return The annotation type information
	*/
	public AnnotationTypeInfo getById(String ID, int itemId)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			AnnotationType at = AnnotationType.getById(dc, itemId);
			return at.toTransferable(new AnnotationTypeInfo());
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	/**
	 	Gets information about annotation types that are available
	 	for an active session. 
	 	
	  	@param ID Id to identify the session.
	  	@param itemType Only return annotation types for a specific item type. The
	  		value is the string representation of {@link Item#name()}.
	  		Use null to return annotation types for all item types
	  	@param qOpt QueryOptions to use when getting the projects
	  	@return Information objects in an array for each found annotation type.
	*/
	public AnnotationTypeInfo[] getAnnotationTypes(String ID, String itemType, QueryOptions qOpt)
	{
		SessionControl sc = getSessionControl(ID);
		DbControl dc = sc.newDbControl();	
		try
		{
			Item item = itemType == null ? null : Item.valueOf(itemType);
			ItemQuery<AnnotationType> query = AnnotationType.getQuery(item);
			query = util.getConfiguredItemQuery(query, qOpt);
			return util.listToInfo(query.list(dc), AnnotationTypeInfo.class);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
